# file       : get_Q_info.py
# author     : Georg Verweyen (OSM=user_5359)
# description: reads the stored result of overpass-api query
#              about wikidata and name and additionally prints 
#              out the information about the name giver and his 
#              gender
#
#              example query for the city Bonn, Germany
#              see https://osm.org/relation/62508
#                 [out:csv("name:etymology:wikidata", name)][maxsize:1000000000][timeout:600];
#                 ( area(3600062508); )->.a;
#                 (
#                     relation["type"="associatedStreet"]["name:etymology:wikidata"](area.a);
#                     relation["type"="street"]["name:etymology:wikidata"](area.a);
#                     relation["type"="multipolygon"]["place"]["name"]["name:etymology:wikidata"](area.a);
#                     relation["type"="multipolygon"]["highway"]["name"]["name:etymology:wikidata"](area.a);
#                     way["highway"]["name"]["highway"!="bus_stop"]["highway"!="service"]["name:etymology:wikidata"](area.a); 
#                 );
#
#              based on the project https://github.com/EqualStreetNames
#
#              feel free to send me your output files to grow the knowledge file in the 
#              next version
#
# version    : 0.2.0
# history    : 
#              0.2.0 2021-03-27 version version with full parameter support
#              0.1.5 2021-03-24 add redirect support
#              0.1.0 2021-03-19 project support of equalstreetnames for Leipzig
# parameter  : see python get_Q_info.py -h
#
from argparse import ArgumentParser
import csv
from datetime import datetime
from os import path
import time
import requests

def get_gender(data):
  if "P21" in data["claims"]: 
    if "datavalue" in data["claims"]["P21"][0]["mainsnak"]:
      data_gender=data["claims"]["P21"][0]["mainsnak"]["datavalue"]["value"]["id"] # ["datavalue"]["value"]["id"]
    else:
      data_gender='-'
  else:
    data_gender='-'
  if data_gender=='Q6581072':
    data_gender='F'
  if data_gender=='Q6581097':
    data_gender='M'
  return data_gender;

def get_type(data):
  if "P31" in data["claims"]: 
    data_type=data["claims"]["P31"][0]["mainsnak"]["datavalue"]["value"]["id"]
  else:
    data_type='?'
  if data_type=='Q5':
    data_type='Person'
  if data_type=='Q95074':
    data_type='Fiktiv'
  if data_type=='Q24354': # Theater
    data_type='Gebäude' 
  if data_type=='Q14073567': 
    data_type='Geschwisterpaar' 
  if data_type=='Q1549591': # Großstadt
    data_type='Ort'
  if data_type=='Q15632617':
    data_type='Fiktiv'
  if data_type=='Q16513881': # nordische Gottheit
    data_type='Gottheit'
  if data_type=='Q16521': # Taxon
    data_type='bio. Gruppe'
  if data_type=='Q178342': # Erzengel
    data_type='Fiktiv'
  if data_type=='Q21070568': # möglicherweise fiktiver Mensch
    data_type='Fiktiv'
  if data_type=='Q253019': # Ortsteil
    data_type='Ort'
  if data_type=='Q28640':
    data_type='Beruf'
  if data_type=='Q4271324':
    data_type='Sagenfigur'
  if data_type=='Q42744322': # Stadt in Deutschland
    data_type='Ort'
  if data_type=='Q486972': # Siedlung
    data_type='Ort'
  if data_type=='Q50386450': # Opernfigur
    data_type='Fiktiv'
  if data_type=='Q577': # Jahr
    data_type='Zeit'
  if data_type=='Q6881511': # Unternehmen
    data_type='Firma'
  if data_type=='Q7860962':
    data_type='Straße'
  if data_type=='Q79423884': # Stadtteil von Leipzig
    data_type='Ort'
  if data_type=='Q134626':   
    data_type='Ort'
  if data_type=='Q1548518':   
    data_type='Ort'
  if data_type=='Q16521':   
    data_type='bio.Gruppe'
  if data_type=='Q16970':   
    data_type='Gebäude'
  if data_type=='Q178561':   
    data_type='Geschichte'
  if data_type=='Q1852178':   
    data_type='Ort'
  if data_type=='Q23413':   
    data_type='Gebäude'
  if data_type=='Q2616791':   
    data_type='Ort'
  if data_type=='Q262166':   
    data_type='Ort'
  if data_type=='Q2983893':   
    data_type='Ort'
  if data_type=='Q3957':   
    data_type='Ort'
  if data_type=='Q4022':   
    data_type='Gewässer'
  if data_type=='Q448801':    
    data_type='Ort'
  if data_type=='Q484170':   
    data_type='Ort'
  if data_type=='Q5119':   
    data_type='Ort'
  if data_type=='Q515':   
    data_type='Ort'
  if data_type=='Q54935786':   
    data_type='Ort'
  if data_type=='Q573344':   
    data_type='Gewässer'
  if data_type=='Q707813':   
    data_type='Ort'
  if data_type=='Q925381':   
    data_type='Ort'
  return data_type;

def get_description(data,language):
  if language in data["descriptions"]:
    data_description=data["descriptions"][language]["value"]
  else:
    data_description='-'
  return data_description;

def get_label(data,language):
  if language in data["labels"]:
    data_label=data["labels"][language]["value"]
  else:
    data_label='-'
  return data_label;

parser = ArgumentParser()
parser.add_argument("-i", "--input", dest="file_input", default='checklist.txt',
                    help="read input file, default checklist.txt", metavar="<input file name>")
parser.add_argument("-k", "--knowledge", dest="file_knowledge",  
                    default='knowledge.txt',
                    help="read knowledge (default knowledge.txt)", metavar="<knowledge file name>")
parser.add_argument("-o", "--output", dest="file_output",
                    help="set file name for output file (default Q_info_<timestamp>.txt)", metavar="<file output>")
parser.add_argument("-v", "--verbose", dest="flag_verbose", action='store_true',
                    help="give same statics")
args = parser.parse_args()

url = 'https://www.wikidata.org/wiki/Special:EntityData/'

statistic= {'starttime': time.time(), 'endtime': time.time(), 'cnt_file': 0, 'cnt_knowledge': 0, \
            'cnt_print': 0, 'cnt_cached': 0}
            
if not path.exists(args.file_input):
  print('Error> Input file {:s} doesn\'t exists!'.format(args.file_input))
  exit()
  
knowledge={}
if path.exists(args.file_knowledge):
  with open(args.file_knowledge) as csvdatei:
    csv_reader_object = csv.DictReader(csvdatei, delimiter='\t', fieldnames=("qcode", "1", "2", "3", "4", "5", "6"))
    knowledge = {}
    for row in csv_reader_object:
      statistic ['cnt_knowledge']+=1
      key=row["qcode"].strip()
      value = row
      knowledge[key]=value
    csvdatei.close()

if not (args.file_output):
  args.file_output='Q_info_'+(datetime.now()).strftime('%Y%m%d_%H%M%S')+'.csv'
f_out = open (args.file_output, 'w')

with open(args.file_input) as csvdatei:
  csv_reader_object = csv.reader(csvdatei, delimiter='\t')
  for row in csv_reader_object:
    statistic ['cnt_file']+=1
    if not not row: # only if a row is filled
      save_query=row[0]
      if (len(row)>1):
        name =row[1]
      else:
        name =''
      for query in save_query.split(';'):
        if (query[0]=='Q'):
          row.clear()
          row.append(query)
          row.append(name)
          if (query in knowledge):
            statistic ['cnt_cached']+=1
            row.append(knowledge[query]["2"])
            row.append(knowledge[query]["3"])
            row.append(knowledge[query]["4"])
            row.append(knowledge[query]["5"])
          else:
            r = requests.get(url+query+'.json')
            if (r.status_code==200):
              first=next(iter(r.json()["entities"]))
              if (first!=query):
                print ('Info>',query,' redirected to',first)
              data = r.json()["entities"][first]
              row.append(get_gender(data))
              row.append(get_type(data))
              row.append(get_label(data,'de'))
              row.append(get_description(data, 'de'))
            else:
              row.append('')
              row.append('')
              row.append('')
              row.append('return-code 400 :( ')
          out=row[0].strip()+'\t'+row[1].strip()+'\t'+row[2].strip()+'\t'+ \
              row[3].strip()+'\t'+row[4].strip()+'\t'+row[5].strip()
          f_out.write (out+'\n')
          statistic ['cnt_print']+=1
  csvdatei.close()
  f_out.close()
statistic ['endtime']= time.time()
if args.flag_verbose:
  print('Info> runtime (sec): {:5.3f}'.format(statistic ['endtime']-statistic ['starttime']))
  print('Info> known Q codes: {:5d}'.format(statistic['cnt_knowledge']))
  print('Info> lines readed : {:5d}'.format(statistic['cnt_file']))
  print('Info> lines printed: {:5d}'.format(statistic['cnt_print']))
  print('Info> lines cached : {:5d}'.format(statistic['cnt_cached']))
  print('Info> ouput file   : {:s}'.format(args.file_output))

